@@ -285,8 +285,13 @@ struct _VteTerminalPrivate {
         VteCharacterReplacement character_replacements[2];  /* charsets in the G0 and G1 slots */
         VteCharacterReplacement *character_replacement;     /* pointer to the active one */
 
+
+        /* Word chars */
+        char *word_char_exceptions_string;
+        gunichar *word_char_exceptions;
+        gsize word_char_exceptions_len;
+
 	/* Selection information. */
-        char *word_char_exceptions;
 	gboolean has_selection;
 	gboolean selecting;
 	gboolean selecting_after_threshold;
@@ -357,7 +362,8 @@ struct _VteTerminalPrivate {
 
 	int mouse_tracking_mode; /* this is of type MouseTrackingMode,
 				    but we need to guarantee its type. */
-	guint mouse_last_button;
+        guint mouse_pressed_buttons;      /* bits 0, 1, 2 resp. for buttons 1, 2, 3 */
+        guint mouse_handled_buttons;      /* similar bitmap for buttons we handled ourselves */
 	long mouse_last_x, mouse_last_y;
 	gboolean mouse_autohide;
 	guint mouse_autoscroll_tag;
